@@ -14,7 +14,7 @@ from fastmcp.client.transports import StreamableHttpTransport
 from agentmake.plugins.uba.lib.BibleBooks import BibleBooks
 from agentmake.plugins.uba.lib.BibleParser import BibleVerseParser
 from agentmake import agentmake, getOpenCommand, getDictionaryOutput, edit_file, edit_configurations, extractText, readTextFile, writeTextFile, getCurrentDateTime, AGENTMAKE_USER_DIR, USER_OS, DEVELOPER_MODE, DEFAULT_TEXT_EDITOR
-from agentmake.utils.files import searchFolder, isExistingPath
+from agentmake.utils.files import searchFolder, isExistingPath, sanitize_filename
 from agentmake.etextedit import launch_async
 from agentmake.utils.manage_package import getPackageLatestVersion
 from rich.console import Console
@@ -31,7 +31,7 @@ if not USER_OS == "Windows":
     import readline  # for better input experience
 
 # set window title
-set_title(f"聖經研讀小夥伴 BibleMate AI")
+set_title("聖經研讀小夥伴 BibleMate AI")
 
 parser = argparse.ArgumentParser(description = f"""BibleMate AI {BIBLEMATE_VERSION} CLI options""")
 # global options
@@ -43,9 +43,8 @@ parser.add_argument("-pe", "--promptengineer", action="store", dest="promptengin
 parser.add_argument("-s", "--steps", action="store", dest="steps", type=int, help="Specify the maximum number of steps allowed.")
 parser.add_argument("-e", "--exit", action="store_true", dest="exit", help="exit after the first response (for single-turn use cases).")
 # mcp options
-parser.add_argument("-t", "--token", action="store", dest="token", help="specify a static token to use for authentication with the MCP server; applicable to command `biblemate` only")
-parser.add_argument("-mcp", "--mcp", action="store", dest="mcp", help=f"specify a custom MCP server to use, e.g. 'http://127.0.0.1:{config.mcp_port}/mcp/'; applicable to command `biblemate` only")
-parser.add_argument("-p", "--port", action="store", dest="port", help=f"specify a port for the MCP server to use, e.g. {config.mcp_port}; applicable to command `biblematemcp` only")
+parser.add_argument("-t", "--token", action="store", dest="token", help="specify a static token to use for authentication with the MCP server")
+parser.add_argument("-mcp", "--mcp", action="store", dest="mcp", help=f"specify a custom MCP server to use, e.g. 'http://127.0.0.1:{config.mcp_port}/mcp/'")
 args = parser.parse_args()
 
 if not sys.stdin.isatty():
@@ -84,6 +83,14 @@ Please provide me with the final answer to my original request based on the work
 
 # Original Request
 """
+TOOL_INSTRUCTION_PROMPT = """Please transform the following suggestions into clear, precise, and actionable instructions."""
+TOOL_INSTRUCTION_SUFFIX = request_chinese_response("""
+
+# Remember
+
+* Provide me with the instructions directly.
+* Do not start your response, like, 'Here are the insturctions ...'
+* Do not ask me if I want to execute the instruction.""")
 
 # other temporary config changes
 if args.light == "true":
@@ -103,13 +110,6 @@ elif args.promptengineer == "false":
 if args.steps:
     config.max_steps = args.steps
 
-def mcp():
-    builtin_mcp_server = os.path.join(os.path.dirname(os.path.realpath(__file__)), "bible_study_mcp.py")
-    user_mcp_server = os.path.join(AGENTMAKE_USER_DIR, "biblemate", "bible_study_mcp.py") # The user path has the same basename as the built-in one; users may copy the built-in server settings to this location for customization. 
-    mcp_script = readTextFile(user_mcp_server if os.path.isfile(user_mcp_server) else builtin_mcp_server)
-    mcp_script = mcp_script.replace("mcp.run(show_banner=False)", f'''mcp.run(show_banner=False, transport="http", host="0.0.0.0", port={args.port if args.port else config.mcp_port})''')
-    exec(mcp_script)
-
 def main():
     asyncio.run(main_async())
 
@@ -205,16 +205,18 @@ def display_info(console, info, title=None, border_style=config.color_info_borde
     console.print(info_panel)
     console.print()
 
-def backup_conversation(messages, master_plan, console=None, storage_path=None):
+def backup_conversation(messages, master_plan, console=None, storage_path=None, title=None):
     """Backs up the current conversation to the user's directory."""
     if len(messages) > len(DEFAULT_MESSAGES) and ((not console) or (console and storage_path) or (console and not storage_path and config.backup_required)):
         # determine storage path
         if not storage_path:
             if console:
                 timestamp = getCurrentDateTime()
-                storage_path = os.path.join(AGENTMAKE_USER_DIR, "biblemate", "chats", timestamp)
+                if title:
+                    timestamp += "_"+sanitize_filename(title)[:50].replace(" ", "_")
+                storage_path = os.path.join(BIBLEMATE_USER_DIR, "chats", timestamp)
             else:
-                storage_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), "temp")
+                storage_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), f"temp_{config.backend}")
         # create directory if not exists
         if not os.path.isdir(storage_path):
             Path(storage_path).mkdir(parents=True, exist_ok=True)
@@ -233,8 +235,7 @@ def backup_conversation(messages, master_plan, console=None, storage_path=None):
             console.save_html(html_file, inline_styles=True, theme=MONOKAI)
         # Inform users of the backup location
         if console:
-            info = f"已經備份到： {storage_path}\n報告已存檔： {html_file}"
-            display_info(console, info)
+            display_info(console, storage_path, title="備份")
 
 def get_border_style():
     if config.agent_mode:
@@ -299,7 +300,7 @@ async def main_async():
             client = Client(transport=transport, timeout=config.mcp_timeout)
     else:
         builtin_mcp_server = os.path.join(os.path.dirname(os.path.realpath(__file__)), "bible_study_mcp.py")
-        user_mcp_server = os.path.join(AGENTMAKE_USER_DIR, "biblemate", "bible_study_mcp.py") # The user path has the same basename as the built-in one; users may copy the built-in server settings to this location for customization. 
+        user_mcp_server = os.path.join(BIBLEMATE_USER_DIR, "bible_study_mcp.py") # The user path has the same basename as the built-in one; users may copy the built-in server settings to this location for customization. 
         mcp_server = user_mcp_server if os.path.isfile(user_mcp_server) else builtin_mcp_server        
         client = Client(mcp_server) # no auth for stdio transport
 
@@ -353,7 +354,7 @@ async def main_async():
         template_pattern = "|".join(template_list)
         template_pattern = f"""^({template_pattern})"""
 
-        user_request = ""
+        original_request = user_request = ""
         master_plan = ""
         messages = deepcopy(DEFAULT_MESSAGES) # set the tone
 
@@ -366,7 +367,7 @@ async def main_async():
                     TextColumn("[progress.description]{task.description}"),
                     transient=True  # This makes the progress bar disappear after the task is done
                 ) as progress:
-                    task_id = progress.add_task((description if description else "思考中 ...")+" [可按 `Ctrl+C` 取消]", total=None)
+                    task_id = progress.add_task((description if description else "思考中 ...")+" [可按 Ctrl+C 取消]", total=None)
                     async_task = asyncio.create_task(process())
                     try:
                         while not async_task.done():
@@ -392,9 +393,9 @@ async def main_async():
                 Manages the async task and the progress bar.
                 """
                 if step_number:
-                    print(f"# 進行步驟 [{step_number}] ... [可按 `Ctrl+C` 取消]")
+                    print(f"# 進行步驟 [{step_number}] ... [可按 Ctrl+C 取消]")
                 else:
-                    print("# 啓動中 ... [可按 `Ctrl+C` 取消]")
+                    print("# 啓動中 ... [可按 Ctrl+C 取消]")
                 # Create the async task but don't await it yet.
                 task = asyncio.create_task(run_tool(tool, tool_instruction))
                 # Await the custom async progress bar that awaits the task.
@@ -404,6 +405,27 @@ async def main_async():
                     task.cancel()
                     await asyncio.sleep(0) # Allow cancellation to propagate
                     raise # Re-raise CancelledError
+            # gnerate title
+            async def generate_title():
+                nonlocal console, original_request
+                if not original_request:
+                    return ""
+                generated_title_output = ""
+                generated_title = ""
+                async def run_prompt_engineering():
+                    nonlocal generated_title_output, generated_title
+                    generated_title_output = agentmake(original_request, system=get_system_generate_longer_title(), **AGENTMAKE_CONFIG)
+                    if generated_title_output:
+                        generated_title = generated_title_output[-1].get("content", "").strip().replace("標題: ", "")
+                try:
+                    await thinking(run_prompt_engineering, "撰寫標題中 ...")
+                    if not generated_title_output:
+                        display_cancel_message(console)
+                except (KeyboardInterrupt, asyncio.CancelledError):
+                    display_cancel_message(console)
+                if generated_title:
+                    set_title(f"聖經研讀小夥伴 📝 {generated_title}")
+                return generated_title
 
             if not APP_START and args.exit:
                 break
@@ -422,7 +444,7 @@ async def main_async():
                     # check connection
                     if not config.skip_connection_check:
                         try:
-                            agentmake("您好！", system=DEFAULT_SYSTEM)
+                            agentmake("您好！", backend=config.backend, system=DEFAULT_SYSTEM)
                         except Exception as e:
                             print("連線失敗！請確認您的網路連線穩定，並確保我 AI 供應端和模型配置正確。")
                             print("開啟 https://github.com/eliranwong/agentmake#supported-backends 以取得更多諮詢。\n")
@@ -470,6 +492,9 @@ async def main_async():
                     user_request = f"//bible/{refs}" if ":" in user_request else f"//chapter/{refs}"
                 else:
                     user_request = f"//search/{user_request}"
+            # direct text response
+            elif user_request.startswith("\\"):
+                user_request = "@get_direct_text_response " + "\n\n" + fix_string(user_request[1:])
             # system commands
             elif user_request.startswith("!"):
                 cmd = user_request[1:].strip()
@@ -697,7 +722,7 @@ async def main_async():
                 else:
                     user_request = f".open {chats_path}"
             elif user_request == ".reload":
-                temp_dir = os.path.join(os.path.dirname(os.path.realpath(__file__)), "temp")
+                temp_dir = os.path.join(os.path.dirname(os.path.realpath(__file__)), f"temp_{config.backend}")
                 last_saved_conversation = os.path.join(temp_dir, "conversation.py")
                 if os.path.isfile(last_saved_conversation):
                     user_request = f".import {temp_dir}"
@@ -725,7 +750,10 @@ async def main_async():
                         print("只接受包含 `conversation.py` 和 `master_plan.md` 的文件夾。")
                         os.chdir(cwd)
                         continue
-                    backup_conversation(messages, master_plan, console)
+                    if config.backup_required:
+                        generated_title = await generate_title()
+                        if generated_title:
+                            backup_conversation(messages, master_plan, console, title=generated_title)
                     config.backup_required = False
                     messages = [{"role": i["role"], "content": i["content"]} for i in eval(readTextFile(file_path)) if i.get("role", "") in ("user", "assistant")]
                     if messages:
@@ -767,7 +795,10 @@ async def main_async():
             # predefined operations with `.` commands
             if user_request in config.action_list:
                 if user_request == ".backup":
-                    backup_conversation(messages, master_plan, console)
+                    if config.backup_required:
+                        generated_title = await generate_title()
+                        if generated_title:
+                            backup_conversation(messages, master_plan, console, title=generated_title)
                     config.backup_required = False
                 elif user_request == ".help":
                     actions = "\n".join([f"- `{k}`: {v}" for k, v in config.action_list.items()])
@@ -1024,10 +1055,15 @@ https://github.com/eliranwong/biblemate
                         write_user_config()
                         display_info(console, f"預設原文字典設定爲 `{config.default_lexicon}`", title="設定")
                 elif user_request in (".new", ".exit"):
-                    backup_conversation(messages, master_plan, console) # backup
+                    # backup before exit / new
+                    if config.backup_required:
+                        generated_title = await generate_title()
+                        if generated_title:
+                            backup_conversation(messages, master_plan, console, title=generated_title)
                     config.backup_required = False
                 # reset
                 if user_request == ".new":
+                    set_title("聖經研讀小夥伴 BibleMate AI")
                     user_request = ""
                     master_plan = ""
                     messages = deepcopy(DEFAULT_MESSAGES)
@@ -1197,9 +1233,43 @@ https://github.com/eliranwong/biblemate
                         messages = agentmake(messages, system="auto", **AGENTMAKE_CONFIG)
                 messages[-1]["content"] = fix_string(messages[-1]["content"])
 
-            # user specify a single tool
+            # execute a single tool
             if specified_tool and not specified_tool == "@@" and not specified_prompt:
-                display_info(console,Markdown(messages[-1]['content']), border_style=get_border_style())
+                if not specified_tool == "get_direct_text_response":
+                    # refine instruction
+                    refined_instruction_output = []
+                    refined_instruction = ""
+                    async def refine_tool_instruction():
+                        nonlocal refined_instruction_output, refined_instruction, tools, messages, original_request, specified_tool
+                        specified_tool_description = tools.get(specified_tool, "No description available.")
+                        instruction_draft = TOOL_INSTRUCTION_PROMPT + "\n\n# Suggestions\n\n"+messages[-1]['content']+f"\n\n# Tool Description of `{specified_tool}`\n\n"+specified_tool_description+TOOL_INSTRUCTION_SUFFIX
+                        system_tool_instruction = get_system_tool_instruction(specified_tool, specified_tool_description)
+                        if config.light:
+                            this_messages = get_lite_messages(messages, original_request)
+                        else:
+                            this_messages = [{"role": "system", "content": system_tool_instruction}]+messages[len(DEFAULT_MESSAGES):]
+                        refined_instruction_output = agentmake(this_messages, system=system_tool_instruction, follow_up_prompt=instruction_draft, **AGENTMAKE_CONFIG)
+                        refined_instruction = refined_instruction_output[-1].get("content", "").strip()
+                    try:
+                        await thinking(refine_tool_instruction, "優化工具指示中 ...")
+                        if not refined_instruction_output:
+                            display_cancel_message(console)
+                            if step == 1:
+                                config.current_prompt = original_request
+                            conversation_broken = True
+                            break
+                        else:
+                            messages[-1]['content'] = refined_instruction
+                    except (KeyboardInterrupt, asyncio.CancelledError):
+                        display_cancel_message(console)
+                        if step == 1:
+                            config.current_prompt = original_request
+                        conversation_broken = True
+                        break
+                    # display refined instruction
+                    display_info(console, Markdown(messages[-1]['content']), title="優化後的指示", border_style=get_border_style())
+                elif config.prompt_engineering:
+                    display_info(console, Markdown(messages[-1]['content']), title="優化後的指示", border_style=get_border_style())
                 try:
                     await process_tool(specified_tool, user_request)
                 except (KeyboardInterrupt, asyncio.CancelledError):
@@ -1402,32 +1472,19 @@ Available tools are: {available_tools}.
                 next_step_output = []
                 next_step = ""
                 async def get_next_step():
-                    nonlocal next_step_output, next_step, next_tool, next_suggestion, tools
+                    nonlocal next_step_output, next_step, next_tool, next_suggestion, tools, messages, original_request
                     if next_tool == "get_direct_text_response":
                         next_step_output = agentmake(next_suggestion, system="biblemate/direct_instruction", **AGENTMAKE_CONFIG)
                         next_step = next_step_output[-1].get("content", "").strip()
                     else:
                         next_tool_description = tools.get(next_tool, "No description available.")
+                        next_suggestion = TOOL_INSTRUCTION_PROMPT + "\n\n# Suggestions\n\n"+next_suggestion+f"\n\n# Tool Description of `{next_tool}`\n\n"+next_tool_description+TOOL_INSTRUCTION_SUFFIX
                         system_tool_instruction = get_system_tool_instruction(next_tool, next_tool_description)
-                        # No context; missing information for the tool from the conversation
-                        #next_step_output = agentmake(next_suggestion, system=system_tool_instruction, **AGENTMAKE_CONFIG)
-                        #next_step = next_step_output[-1].get("content", "").strip()
-                        # Full context, but when a conversation goes long, the agent loses track of the system message.
-                        #next_step = agentmake([{"role": "system", "content": system_tool_instruction}]+messages[len(DEFAULT_MESSAGES):], follow_up_prompt=next_suggestion, **AGENTMAKE_CONFIG)[-1].get("content", "").strip()
-                        # Minimum context that includes the original request and the latest exchanges, if any
-                        lite_messages = get_lite_messages(messages, original_request)
-                        next_suggestion += f"""
-
-# Remember:
-* Do NOT provide the answer or perform the task. Provide the instruction ONLY, which the AI assistant will follow or answer.
-* You are here to proved the instruction for the current step ONLY.
-* Do not mention the tool name in your instruction.
-* Do not mention further steps or tools to be used after this instruction.
-* Only provide the instruction for the specified tool `{next_tool}`.
-* Pay attention to the information the tool requires and provide the necessary details in your instruction.
-
-You provide the converted instruction directly, without any additional commentary or explanation."""
-                        next_step_output = agentmake(lite_messages, system=system_tool_instruction, follow_up_prompt=next_suggestion, **AGENTMAKE_CONFIG)
+                        if config.light:
+                            this_messages = get_lite_messages(messages, original_request)
+                        else:
+                            this_messages = [{"role": "system", "content": system_tool_instruction}]+messages[len(DEFAULT_MESSAGES):]
+                        next_step_output = agentmake(this_messages, system=system_tool_instruction, follow_up_prompt=next_suggestion, **AGENTMAKE_CONFIG)
                         next_step = next_step_output[-1].get("content", "").strip()
                 try:
                     await thinking(get_next_step, "撰寫下一步指示中 ...")
@@ -1526,7 +1583,10 @@ You provide the converted instruction directly, without any additional commentar
             # Backup
             if not conversation_broken:
                 print()
-                backup_conversation(messages, master_plan, console)
+                if config.backup_required:
+                    generated_title = await generate_title()
+                    if generated_title:
+                        backup_conversation(messages, master_plan, console, title=generated_title)
                 config.backup_required = False
     
     # back up configurations
